<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:UML="org.omg.xmi.namespace.UML">

<xsl:variable name="lcletters">abcdefghijklmnopqrstuvwxyz</xsl:variable>
<xsl:variable name="ucletters">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>
<xsl:variable name="ProfileFile">orderedProfile.xml</xsl:variable>
<xsl:variable name="EPAGESProfile">http://argouml.org/user-profiles/EPAGES.xmi#</xsl:variable>

<xsl:output method="xml" encoding="ISO-8859-1" indent="yes"/>

<xsl:template match="/">
	<Model>
	<xsl:attribute name="Name"><xsl:value-of select="XMI/XMI.content/UML:Model/@name" /></xsl:attribute>
	<xsl:attribute name="OrderID">1</xsl:attribute>
	<xsl:for-each select="XMI/XMI.content/UML:Model/UML:Namespace.ownedElement" >

		<xsl:apply-templates select="UML:Package"/>

		<xsl:apply-templates select="UML:Class" />

	</xsl:for-each>
	</Model>
</xsl:template>

<xsl:template match="UML:Package">
	<xsl:if test="@name">
		<RefModel>
			<xsl:attribute name="Name"><xsl:value-of select="substring-after(@name, '/')" /></xsl:attribute>
			<xsl:attribute name="Cartridge"><xsl:value-of select="substring-before(@name, '/')" /></xsl:attribute>
		</RefModel>
	</xsl:if>
</xsl:template>

<xsl:template match="UML:Class">
	<xsl:variable name="TableID"><xsl:call-template name="getIDOfTable" /></xsl:variable>
	<xsl:variable name="ViewID"><xsl:call-template name="getIDOfView" /></xsl:variable>
	<xsl:if test="substring-after(substring(substring(UML:ModelElement.stereotype/UML:Stereotype/@href,string-length($EPAGESProfile)+1), 41),':') = substring-after($TableID,'epages')" >

		<Table>
			<xsl:attribute name="Name"><xsl:value-of select="@name" /></xsl:attribute>
			<xsl:attribute name="Code"><xsl:value-of select="translate(@name,$ucletters,$lcletters)" /></xsl:attribute>

			<xsl:apply-templates select="UML:GeneralizableElement.generalization" /> <!--the Table is an ObjectTable -->

			<xsl:apply-templates select="UML:ModelElement.taggedValue/UML:TaggedValue/UML:TaggedValue.dataValue" /> <!-- import the Comment as an Table-Attribute -->

			<xsl:apply-templates select="UML:Classifier.feature/UML:Attribute" />

			<xsl:call-template name="setPK"/>

			<xsl:call-template name="setFKs"/>

			<xsl:call-template name="setIndexes" />

		</Table>
	</xsl:if>
	<xsl:if test="substring-after(substring(substring(UML:ModelElement.stereotype/UML:Stereotype/@href,string-length($EPAGESProfile)+1), 41),':') = substring-after($ViewID,'epages')" >

		<View>
			<xsl:attribute name="Name"><xsl:value-of select="@name" /></xsl:attribute>
			<xsl:attribute name="Code"><xsl:value-of select="translate(@name,$ucletters,$lcletters)" /></xsl:attribute>
			<SQLQuery>
				<xsl:value-of select="UML:ModelElement.taggedValue/UML:TaggedValue/UML:TaggedValue.dataValue" />
			</SQLQuery>

		</View>
	</xsl:if>

</xsl:template>

<xsl:template match="UML:GeneralizableElement.generalization">

	<xsl:variable name="GeneralizationID"><xsl:value-of select="UML:Generalization/@xmi.idref"/></xsl:variable>
	<xsl:variable name="ParentID" select="../../UML:Generalization[@xmi.id = $GeneralizationID]/UML:Generalization.parent/UML:Class/@xmi.idref" />
	<xsl:choose>
		<xsl:when test="../../UML:Generalization[@xmi.id = $GeneralizationID]/UML:ModelElement.stereotype" > <!-- the ObjectTable is an AttributeTable -->
			<xsl:attribute name="AttributeClass">
				<xsl:choose>
					<xsl:when test="../../UML:Class[@xmi.id = $ParentID]">
						<xsl:value-of select="../../UML:Class[@xmi.id = $ParentID]/@name" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="../../UML:Package/UML:Namespace.ownedElement/UML:Class[@xmi.id = $ParentID]/@name" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		</xsl:when>
		<xsl:otherwise>  <!-- ObjectTable inherit from BaseClass -->
			<xsl:attribute name="BaseClass">
				<xsl:choose>
					<xsl:when test="../../UML:Class[@xmi.id = $ParentID]">
						<xsl:value-of select="../../UML:Class[@xmi.id = $ParentID]/@name" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="../../UML:Package/UML:Namespace.ownedElement/UML:Class[@xmi.id = $ParentID]/@name" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		</xsl:otherwise>
	</xsl:choose>

</xsl:template>

<xsl:template match="UML:Classifier.feature/UML:Attribute" >

	<Column>
		<xsl:attribute name="Name"><xsl:value-of select="@name" /></xsl:attribute>
		<xsl:attribute name="Code"><xsl:value-of select="translate(@name,$ucletters,$lcletters)" /></xsl:attribute>
		<xsl:attribute name="Type">
				<xsl:if test="starts-with(UML:StructuralFeature.type/UML:DataType/@href, $EPAGESProfile)">
					<xsl:variable name="TypeID"><xsl:value-of select="concat('epages', substring-after(substring(substring(UML:StructuralFeature.type/UML:DataType/@href,string-length($EPAGESProfile)+1), 41),':'))" /></xsl:variable>
					<xsl:value-of select="document($ProfileFile)/Profile/DataTypes/*[name() = $TypeID]" />
				</xsl:if>

		</xsl:attribute>

		<!-- set the properties from the columns, which are results of the stereotypes -->

		<xsl:for-each select="UML:ModelElement.stereotype/UML:Stereotype">
			<xsl:variable name="StereotypeID"><xsl:value-of select="concat('epages', substring-after(substring(substring(@href,string-length($EPAGESProfile)+1), 41),':'))" /></xsl:variable>
			<xsl:variable name="Stereotype"><xsl:value-of select="document($ProfileFile)/Profile/Stereotypes/*[name() = $StereotypeID]" /></xsl:variable>

			<xsl:if test="$Stereotype = 'NotNull' ">
				<xsl:attribute name="IsNotNull">true</xsl:attribute>
			</xsl:if>

			<xsl:if test="$Stereotype = 'PK' ">
				<xsl:attribute name="IsNotNull">true</xsl:attribute>
				<xsl:attribute name="IsReadOnly">true</xsl:attribute>
			</xsl:if>

			<xsl:if test="$Stereotype = 'AutoIncr'">
				<xsl:attribute name="AutoIncr">true</xsl:attribute>
			</xsl:if>

			<xsl:if test="$Stereotype = 'AFCreation'">
				<xsl:attribute name="AutoFill">CreationDate</xsl:attribute>
			</xsl:if>

			<xsl:if test="$Stereotype = 'AFChange'">
				<xsl:attribute name="AutoFill">ChangeDate</xsl:attribute>
			</xsl:if>

			<xsl:if test="$Stereotype = 'FK'">
				<xsl:call-template name="setObjectClass" />
			</xsl:if>
		</xsl:for-each>

		<xsl:apply-templates select="UML:ModelElement.taggedValue/UML:TaggedValue/UML:TaggedValue.dataValue" /> <!-- import the Comment as an Table-Attribute -->

	</Column>

</xsl:template>

<xsl:template match="UML:ModelElement.taggedValue/UML:TaggedValue/UML:TaggedValue.dataValue" >
	<xsl:attribute name="Comment"><xsl:value-of select="text()" /></xsl:attribute>
</xsl:template>

<xsl:template name="setObjectClass">
	<xsl:variable name="AttributeName" ><xsl:value-of select="../../@name" /></xsl:variable>
	<xsl:variable name="ChildClassID" ><xsl:value-of select="../../../../@xmi.id" /></xsl:variable>
	<xsl:variable name="ClassID" >
		<xsl:for-each select="../../../../../UML:Generalization[starts-with(@name, $AttributeName)]">
			<xsl:if test="UML:Generalization.child/UML:Class/@xmi.idref = $ChildClassID">
				<xsl:value-of select="UML:Generalization.parent/UML:Class/@xmi.idref" />
			</xsl:if>
		</xsl:for-each>

		<xsl:for-each select="../../../../../UML:Association[starts-with(@name, $AttributeName)]">
			<xsl:if test="UML:Association.connection/UML:AssociationEnd[@isNavigable = 'false']/UML:AssociationEnd.participant/UML:Class/@xmi.idref = $ChildClassID">
				<xsl:value-of select="UML:Association.connection/UML:AssociationEnd[@isNavigable = 'true']/UML:AssociationEnd.participant/UML:Class/@xmi.idref" />
			</xsl:if>
		</xsl:for-each>

	</xsl:variable>
	<xsl:variable name="ClassName" >
		<xsl:value-of select="../../../../../UML:Class[@xmi.id = $ClassID]/@name" />
		<xsl:value-of select="../../../../../UML:Package/UML:Namespace.ownedElement/UML:Class[@xmi.id = $ClassID]/@name" />
	</xsl:variable>

	<xsl:choose>
		<xsl:when test="../../UML:Attribute.initialValue/UML:Expression/@language = 'ObjectAttribute'">
			<xsl:attribute name="ObjectAttribute"><xsl:value-of select="../../UML:Attribute.initialValue/UML:Expression/@body" /></xsl:attribute>
			<xsl:if test="../../UML:Attribute.initialValue/UML:Expression/@body != $ClassName" >
				<xsl:attribute name="ObjectClass"><xsl:value-of select="$ClassName" /></xsl:attribute>
			</xsl:if >
		</xsl:when >
		<xsl:otherwise>
			<xsl:attribute name="ObjectAttribute"><xsl:value-of select="$ClassName" /></xsl:attribute>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


<xsl:template name="setPK">
	<PrimaryKey>
		<xsl:attribute name="Name"><xsl:value-of select="concat('pk_', translate(@name,$ucletters,$lcletters))" /></xsl:attribute>
		<xsl:variable name="PKID"><xsl:call-template name="getIDOfPK"/></xsl:variable>
		<xsl:for-each select="UML:Classifier.feature/UML:Attribute/UML:ModelElement.stereotype/UML:Stereotype[substring-after(substring(substring(@href,string-length($EPAGESProfile)+1), 41),':') = substring-after($PKID,'epages')]" >
			<Attribute>
				<xsl:attribute name="Name"><xsl:value-of select="../../@name"/></xsl:attribute>
			</Attribute>
		</xsl:for-each>
	</PrimaryKey>
</xsl:template>

<xsl:template name="setFKs">

	<xsl:call-template name="FKsFromGeneralizations" />

	<xsl:call-template name="FKsFromAssociations" />

</xsl:template>


<xsl:template name="FKsFromGeneralizations" >
	<xsl:variable name="TableID"><xsl:value-of select="@xmi.id"/></xsl:variable>
	<xsl:variable name="ChildTable"><xsl:value-of select="@name" /></xsl:variable>

	<xsl:for-each select="../UML:Generalization/UML:Generalization.child/UML:Class[@xmi.idref = $TableID]">

		<xsl:variable name="ParentID" ><xsl:value-of select="../../UML:Generalization.parent/UML:Class/@xmi.idref" /></xsl:variable>
		<xsl:variable name="ParentTable"><xsl:value-of select="../../../UML:Class[@xmi.id = $ParentID]/@name" /><xsl:value-of select="../../../UML:Package/UML:Namespace.ownedElement/UML:Class[@xmi.id = $ParentID]/@name" /></xsl:variable>
		<xsl:variable name="ChildColumn"><xsl:value-of select="substring-before(../../@name , ' = ')" /></xsl:variable>
		<xsl:variable name="ParentColumn"><xsl:value-of select="substring-after(../../@name , ' = ')" /></xsl:variable>

		<ForeignKey>
			<xsl:attribute name="Name" ><xsl:value-of select="concat('fk_',translate($ChildTable,$ucletters,$lcletters),'_',translate($ChildColumn,$ucletters,$lcletters))" /></xsl:attribute>
			<xsl:attribute name="RefTable" ><xsl:value-of select="$ParentTable" /></xsl:attribute>
			<Reference>
				<xsl:attribute name="Attribute"><xsl:value-of select="$ChildColumn" /></xsl:attribute>
				<xsl:attribute name="RefAttribute"><xsl:value-of select="$ParentColumn" /></xsl:attribute>
			</Reference>
		</ForeignKey>

	</xsl:for-each>
</xsl:template>

<xsl:template name="FKsFromAssociations" >
	<xsl:variable name="TableID"><xsl:value-of select="@xmi.id"/></xsl:variable>
	<xsl:variable name="ChildTable"><xsl:value-of select="@name" /></xsl:variable>

	<xsl:for-each select="../UML:Association/UML:Association.connection/UML:AssociationEnd[@isNavigable = 'false']/UML:AssociationEnd.participant/UML:Class[@xmi.idref = $TableID]">

		<xsl:variable name="ParentID" ><xsl:value-of select="../../../UML:AssociationEnd[@isNavigable = 'true']/UML:AssociationEnd.participant/UML:Class/@xmi.idref" /></xsl:variable>
		<xsl:variable name="ParentTable"><xsl:value-of select="../../../../../UML:Class[@xmi.id = $ParentID]/@name" /><xsl:value-of select="../../../../../UML:Package/UML:Namespace.ownedElement/UML:Class[@xmi.id = $ParentID]/@name" /></xsl:variable>
		<xsl:variable name="ChildColumn"><xsl:value-of select="substring-before(../../../../@name , ' = ')" /></xsl:variable>
		<xsl:variable name="ParentColumn"><xsl:value-of select="substring-after(../../../../@name , ' = ')" /></xsl:variable>
		<ForeignKey>
			<xsl:attribute name="Name" ><xsl:value-of select="concat('fk_',translate($ChildTable,$ucletters,$lcletters),'_',translate($ChildColumn,$ucletters,$lcletters))" /></xsl:attribute>
			<xsl:attribute name="RefTable" ><xsl:value-of select="$ParentTable" /></xsl:attribute>
			<Reference>
				<xsl:attribute name="Attribute"><xsl:value-of select="$ChildColumn" /></xsl:attribute>
				<xsl:attribute name="RefAttribute"><xsl:value-of select="$ParentColumn" /></xsl:attribute>
			</Reference>
		</ForeignKey>

	</xsl:for-each>
</xsl:template>

<xsl:template name="setIndexes">
	<xsl:variable name="IndexID"><xsl:call-template name="getIDOfIndex"/></xsl:variable>
	<xsl:variable name="UniqueID"><xsl:call-template name="getIDOfUnique"/></xsl:variable>
	<xsl:for-each select="UML:Classifier.feature/UML:Operation/UML:ModelElement.stereotype/UML:Stereotype" >
		<xsl:if test="substring-after(substring(substring(@href,string-length($EPAGESProfile)+1), 41),':') = substring-after($IndexID,'epages')" >
			<Index>
				<xsl:attribute name="Name"><xsl:value-of select="../../@name" /></xsl:attribute>
				<xsl:attribute name="IsUnique">false</xsl:attribute >
				<xsl:for-each select="../../UML:BehavioralFeature.parameter/UML:Parameter[@name != 'return']" >
					<Attribute>
						<xsl:attribute name="Name"><xsl:value-of select="@name"/></xsl:attribute>
					</Attribute>
				</xsl:for-each>
			</Index>
		</xsl:if>
		<xsl:if test="substring-after(substring(substring(@href,string-length($EPAGESProfile)+1), 41),':') = substring-after($UniqueID,'epages')" >
			<Index>
				<xsl:attribute name="Name"><xsl:value-of select="../../@name" /></xsl:attribute>
				<xsl:attribute name="IsUnique">true</xsl:attribute>
				<xsl:for-each select="../../UML:BehavioralFeature.parameter/UML:Parameter[@name != 'return']" >
					<Attribute>
						<xsl:attribute name="Name"><xsl:value-of select="@name"/></xsl:attribute>
					</Attribute>
				</xsl:for-each>
			</Index>
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="getIDOfTable">
	<xsl:for-each select="document($ProfileFile)/Profile/Stereotypes/*">
		<xsl:if test="text() = 'Table'">
			<xsl:value-of select="name()" />
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="getIDOfView">
	<xsl:for-each select="document($ProfileFile)/Profile/Stereotypes/*">
		<xsl:if test="text() = 'View'">
			<xsl:value-of select="name()" />
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="getIDOfPK">
	<xsl:for-each select="document($ProfileFile)/Profile/Stereotypes/*">
		<xsl:if test="text() = 'PK'">
			<xsl:value-of select="name()" />
		</xsl:if>
	</xsl:for-each>
</xsl:template>


<xsl:template name="getIDOfIndex">
	<xsl:for-each select="document($ProfileFile)/Profile/Stereotypes/*">
		<xsl:if test="text() = 'Index'">
			<xsl:value-of select="name()" />
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="getIDOfUnique">
	<xsl:for-each select="document($ProfileFile)/Profile/Stereotypes/*">
		<xsl:if test="text() = 'Unique'">
			<xsl:value-of select="name()" />
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="getIDOfROOT">
	<xsl:for-each select="document($ProfileFile)/Profile/Stereotypes/*">
		<xsl:if test="text() = 'ROOT'">
			<xsl:value-of select="name()" />
		</xsl:if>
	</xsl:for-each>
</xsl:template>

</xsl:stylesheet>