/*
 * @copyright   © Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module      de_epages.externalcontent.gadget.rss
 *
 * @revision    $Revision: 1.1.2.1 $
 */

jQuery.plugin('de_epages.externalcontent.gadget.rss', {
	plugin : ['ep.core', 'de_epages.core', 'jQuery.tmpl', 'jQuery.ui.widget'],
	tmpl : ['de_epages.externalcontent.gadget.rss']
}, function ($) {

		// Gagdet class name
	var gClassNames = 'ep-gadget-rss',

		// Class names for template layout
		layoutClassNames = {
			'box' : 'box',
			'default' : 'default'
		},

		// Icon path
		rssIcon = ep.config.storeRoot + '/SF/Gadgets/rss-icon.png',

		// Cache deferred object (AJAX)
		deferred;

	// The actual widget.
	$.widget('de_epages.externalcontent.gadget.Rss', {

		options : {
			url : '',
			// (Valid) url to (valid) rss feed.
			onlyHeadlines : false,
			// If true, display only headlines/titles of entries.
			onlySnippets : false,
			// If true, display only a snippet version (<120 characters) of an entry.
			showImages : true,
			// If false, remove *img* tags from entry's content.
			numberOfEntries : null,
			// Integer specifying the number of entries shown.
			// If null, all (<= 100) available entries will be shown.
			layout : 'default'
			// String specifying the layout template used to render the feed.
			// possible values are 'box' and 'default'.
		},

		_create : function () {
			var self = this,
				o = self.options;

			// Load Google Loader API.
			if( !deferred ){
				if( window.google && window.google.load ){
					deferred = $.Deferred().resolve();
				}
				else{
					deferred = $.ajax({
						url : 'https://www.google.com/jsapi',
						dataType : "script",
						cache : true // We would not want to load the same *script* twice.
					});
				}
			}

			// Load Google Feed API.
			deferred.done(function () {
				google.load('feeds', '1', {callback : $.proxy(self._createFeed, self)});
			});
		},

		_createFeed : function () {
			var o = this.options,
			// Create *feed* object with url *o.url*.
			feed = new google.feeds.Feed(o.url);

			// Return feed entries stored by Google that are no longer in the feed XML.
			feed.includeHistoricalEntries();
			// Set the number of feed entries loaded by this feed to *o.numberOfEntries*. -1 means that all (<= 100) available entries will be shown.
			feed.setNumEntries(o.numberOfEntries ? o.numberOfEntries : -1);
			// Finally, load the entries.
			feed.load($.proxy(this._onFeedLoad, this));
		},

		_onFeedLoad : function (result) {
			var self = this,
				o = self.options;

			if (result.error) {
				self.destroy();
				return;
			}
			else {
				var renderedTemplate = self.renderedTemplate = $.tmpl('de_epages.externalcontent.gadget.rss', result.feed, {
					rssIcon : rssIcon,
					onlySnippets : o.onlySnippets,
					onlyHeadlines : o.onlyHeadlines,
					layout : layoutClassNames[o.layout],
					gClassNames : gClassNames
				});
				if (!o.showImages) {
					renderedTemplate.find('img').not('[src="' + rssIcon + '"]').remove();
				}

				renderedTemplate.appendTo(this.element);

				// Move css link tag to head
				renderedTemplate.tmplItem('elements').style.appendTo('head');
			}
		},

		destroy : function () {
			var self = this;
			// Remove all traces of the widget.
			if (self.renderedTemplate) {
				self.renderedTemplate.remove();
			}
			self.Inherited(arguments);
		}

	});

});