/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.thumbbox
 *
 * @revision		$Revision: 1.1.2.1 $
 */

jQuery.plugin( "ep.ui.thumbbox", {
	plugin:	["ep.ui.core", "jQuery.tmpl", "jQuery.fn.class", "jQuery.ui.widget"],
	tmpl:	["ep.ui.thumbbox"]
}, function( $, undefined ){

	$.widget( "ep.ui.thumbbox", {
		options: {
		//	action			: "lightbox",
			event			: "click",
			images			: "img",			// selector / array of data / data / image node / array of image nodes / jQuery object		//
			width			: 100,				// width of thumbnail
			height			: 100,				// height of thumbnail
			view			: "list"			// style of thumbbox ("list" / "slide" / "fisheye")
		},

		_create: function(){
			var self	= this,
				o		= self.options,
				imgs	= self.images = ep.ui.imgData(o.images),
				width	= o.width,
				height	= o.height,
				srcType	= 'srcXs';

			if( width > 190 || height > 190 ){
				srcType	= 'srcL';
			}
			else if( width > 140 || height > 140 ){
				srcType	= 'srcM';
			}
			else if( width > 90 || height > 90 ){
				srcType	= 'srcS';
			}

			$.extend( self, $.tmpl( 'ep.ui.thumbbox', [{imgData:imgs.get(),srcType:srcType}] ).appendTo(self.element).tmplItem('elements') );

			self._setOptions(o);

			// set current the thumbnail of the current image active
			self.element.find('li:first').addClass('active');
			imgs.on('change', function(){
				var currentSrc = imgs.current()[srcType],
					thumbList = $('img[src="'+currentSrc+'"]').closest('ul'),
					thumbItem = $('img[src="'+currentSrc+'"]').closest('li'),
					thumbboxWidth = $('img[src="'+currentSrc+'"]').closest('.ep-uiThumbbox').width(),
					thumblistLeft = thumbList[0].offsetLeft,
					thumbLeft = thumbItem[0].offsetLeft;

				self.element.find('li.active').removeClass('active');
				thumbItem.addClass('active');

				if(self.options.view === "slide"){
					// scroll active thumb into view if necessary
					if(thumblistLeft + thumbLeft < 0){
						thumbList.stop(true, true).animate({
							'left': -thumbLeft + "px"
						});
					}else if(thumblistLeft + thumbLeft > thumbboxWidth){
						thumbList.stop(true, true).animate({
							'left': (thumbboxWidth-thumbLeft-thumbItem.outerWidth()) + "px"
						});
					}
				}
			});

		},

		_init: function(){

		},

		destroy: function(){
			var self = this;

			self.images.off(".uiThumbbox");
			self.container.remove();
			self.Inherited(arguments);
		},

		_setOption: function( key, value ) {
			var self = this,
				o = self.options,
				imgs = self.images;

			if ( key === "view" ) {
				// reset slide
				self.container
					.off("mousemove.uiThumbbox")
					.css( "height", "" )
					.removeClass(/ep-uiThumbbox-[\w]+/)
					.addClass("ep-uiThumbbox-"+value);

				if( value == "slide" ){
					// init slide
					self.container
						.on("mousemove.uiThumbbox",self.Proxy("_slideMove"))
						.height( self.list.outerHeight(true) );
				}
			}
			else if( key === "width" ){
				self.list
					.children()
					.width(value);
			}
			else if( key === "height" ){
				self.list
					.children()
					.height(value)
					.css("line-height",value+"px");
				self._setOption("view",self.options.view);
			}
			else if( key === "event" ){
				self.element
					.off(".uiThumbbox", "img")
					.on(value + ".uiThumbbox", "img", function(event){
						var item = $(this).data("item");

						if( item !== undefined && item != imgs.currentItem ){

							imgs.current( item );

							if( o.action ){
								imgs.trigger(o.action);
							}
						}
					});
			}

			return this.Inherited(arguments);
		},

		_slideMove: function( event ){
			var self			= this,
				containerLeft	= self.container.offset().left,
				containerWidth	= self.container.width(),
				listWidth		= self.list.width(),
				mouseLeft		= event.pageX - containerLeft,
				speed			= self.list.position().left

			self.list.stop(true);

			if( mouseLeft < (containerWidth * 1/4) ){
				self.list.animate({left: "0px"}, Math.abs(speed*2), 'linear');
			}
			else if( mouseLeft > (containerWidth * 3/4) ){
				self.list.animate({left: (containerWidth-listWidth)+"px"}, Math.abs((listWidth-containerWidth+speed)*2), 'linear');
			}
		}
	});

});
