use strict;
use Test::More;
use Perl::Critic;
use DE_EPAGES::Core::API::Script qw( RunScript );
use lib ("$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Object/Scripts");

use Perl::Critic::Policy::DE_EPAGES::Security::ProhibitLoadObjectInUI; ##no critic(RequireCorrectImports)
# Tests Perl::Critic::Policy::DE_EPAGES::Security::ProhibitLoadObjectInUI

sub Main {

    my $critic = Perl::Critic->new(
        -theme    => 'epages',
        -severity => 5,
        -profile  => 'NONE',
    );
    ok( defined $critic, 'Perl::Critic initialized' );

    $critic->add_policy( -policy => 'DE_EPAGES::Security::ProhibitLoadObjectInUI' );
    my $policyLoaded = scalar grep { $_ =~ 'DE_EPAGES::Security::ProhibitLoadObjectInUI' }
    $critic->policies();
    ok( $policyLoaded, 'Policy loaded' );

    ##no critic qw(ProhibitImplicitNewlines ProhibitMissingImports ProhibitUnusedVariables RequireInterpolationOfMetachars)
    my $sourceCode = q(
    package T::UI::Bla;
        statement();
        my $alles_prima = 1;
    );
    ok( !$critic->critique( \$sourceCode ), 'funtion not found' );

    $sourceCode = q(
    package T::UI::Bla;
        $site->loadObject(12345);
    );
    ok( !$critic->critique( \$sourceCode ), 'function site->loadObject used' );

    $sourceCode = q(
    package T::XX::Bla;
        $site->loadObject(12345);
    );
    ok( !$critic->critique( \$sourceCode ), 'function LoadObject used outside of UI Package' );

    $sourceCode = q(
    package T::UI::Bla;
    use Bla qw ( LoadObject );
    );
    ok( !$critic->critique( \$sourceCode ), 'LoadObject import should be ok.' );

    $sourceCode = q(
    package T::UI::Bla;
        LoadObject(123);
    );
    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, 'LoadObject used in UI package. Alarm.' ) or diag(@violations);

    done_testing();
}

RunScript( Sub => \&Main );
