use strict;
use Test::Simple tests => 4;
use DE_EPAGES::Core::API::Error qw( Error ExistsError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias ExistsClassByAlias Transaction );

sub Main {
    ConnectTestSite();

    ok( Init(), 'initialization' );

    my $ClassObject = LoadClassByAlias( 'Object' );

    DBI->trace(1,$ENV{'EPAGES_LOG'}.'/dbi_trace.txt');

    eval {
        Transaction (sub {

            my $Class1 = $ClassObject->insertSubClass( { Alias => 'C1' } );
            ok( $Class1->id, 'insertSubClass 1' );

            Error('TestTransaction');

            my $Class2 = $ClassObject->insertSubClass( { Alias => 'C2' } );
            ok( $Class2->id, 'insertSubClass 2' );
        },'TestTransaction');
    };
    ok( ExistsError(), 'Transaction was aborted after first insert' );
    DBI->trace(0,$ENV{'EPAGES_LOG'}.'/dbi_trace.txt');

    ok ( (not grep { $_->alias eq 'C1' }  @{$ClassObject->subClasses}), 'C1 not found');
    ok( Cleanup(), 'cleanup' );

    DisconnectSite();
}

sub Cleanup {
    foreach my $ClassName ( qw( C1 C2 C3 C4 ) ) {
        next unless ExistsClassByAlias( $ClassName );
        LoadClassByAlias( $ClassName )->delete;
    }

    return 1;
}

sub Init {
    Cleanup();
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
