use Test::More;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( ConnectToSite DisconnectSite );
use DE_EPAGES::Trigger::API::Table::Hook qw ( DeleteHook InsertHook GetPKeyHooksByName );
use DE_EPAGES::Trigger::API::Table::HookFunction qw ( InsertHookFunction );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

our $CALLED = 0;

sub Main {
    plan tests => 6;
    ConnectToSite();
    Cleanup();
    Test_Package1();
    Cleanup();
    DisconnectSite();
}

sub Test_Package1 {
    eval {
        my $HookID = InsertHook({'Name'=>'TestHook'});
        like($HookID, qr/^\d+$/, 'HookID is numeric');
        InsertHookFunction({
            'HookID' => $HookID,
            'OrderNo' => 1,
            'FunctionName' => 'TestHookProc',
        });
        my $HookID2 = InsertHook({'Name'=>'TestHookPackage'});
        like($HookID2, qr/^\d+$/, 'HookID2 is numeric');
        InsertHookFunction({
            'HookID' => $HookID2,
            'OrderNo' => 1,
            'FunctionName' => 'DE_EPAGES::Trigger::t::HookT::TestHookProc',
        });
        TriggerHook('TestHook', { 'TestParameter'=>'Param' });
        is($CALLED, 1, 'TestHook triggered call to HookFunction');

        # test repeated calls
        TriggerHook('TestHook', { 'TestParameter'=>'Param' });
        TriggerHook('TestHook', { 'TestParameter'=>'Param' });
        is($CALLED, 3, 'TestHook triggered repeated calls to HookFunction');

        $DE_EPAGES::Trigger::t::HookT::CALLED = 0;
        TriggerHook('TestHookPackage', { 'TestParameter'=>'Param' });
        is($DE_EPAGES::Trigger::t::HookT::CALLED, 1, 'TestHookPackage triggered call to HookT function');
    };
    if (ExistsError()) {
        my $Error = GetError();
        fail($Error->longMessage());
    } else {
        pass('Test_Package1 passed');
    }
}

sub TestHookProc {
    my ($args) = @_;
    Error('NO_HOOK_NAME') unless defined $args->{'HookName'}      and $args->{'HookName'} eq 'TestHook';
    Error('NO_HOOK_PARA') unless defined $args->{'TestParameter'} and $args->{'TestParameter'} eq 'Param';
    ++$CALLED;
}

sub Cleanup {
    my $IDs = GetPKeyHooksByName('TestHook');
    push @$IDs, @{GetPKeyHooksByName('TestHookPackage')};
    DeleteHook($_) foreach @$IDs;
}

RunScript(
    'Debug' => 1,
    'Sub' => \&Main,
);

1;

